@@ -1,14 +1,20 @@
-import jetbrains.buildServer.configs.kotlin.v2019_2.*
-import jetbrains.buildServer.configs.kotlin.v2019_2.buildFeatures.PullRequests
-import jetbrains.buildServer.configs.kotlin.v2019_2.buildFeatures.commitStatusPublisher
-import jetbrains.buildServer.configs.kotlin.v2019_2.buildFeatures.golang
-import jetbrains.buildServer.configs.kotlin.v2019_2.buildFeatures.pullRequests
-import jetbrains.buildServer.configs.kotlin.v2019_2.buildSteps.dockerCommand
-import jetbrains.buildServer.configs.kotlin.v2019_2.buildSteps.exec
-import jetbrains.buildServer.configs.kotlin.v2019_2.buildSteps.powerShell
-import jetbrains.buildServer.configs.kotlin.v2019_2.failureConditions.BuildFailureOnMetric
-import jetbrains.buildServer.configs.kotlin.v2019_2.failureConditions.failOnMetricChange
-import jetbrains.buildServer.configs.kotlin.v2019_2.triggers.vcs
+import jetbrains.buildServer.configs.kotlin.AbsoluteId
+import jetbrains.buildServer.configs.kotlin.BuildType
+import jetbrains.buildServer.configs.kotlin.DslContext
+import jetbrains.buildServer.configs.kotlin.Project
+import jetbrains.buildServer.configs.kotlin.buildFeatures.PullRequests
+import jetbrains.buildServer.configs.kotlin.buildFeatures.commitStatusPublisher
+import jetbrains.buildServer.configs.kotlin.buildFeatures.golang
+import jetbrains.buildServer.configs.kotlin.buildFeatures.pullRequests
+import jetbrains.buildServer.configs.kotlin.buildSteps.dockerCommand
+import jetbrains.buildServer.configs.kotlin.buildSteps.exec
+import jetbrains.buildServer.configs.kotlin.buildSteps.powerShell
+import jetbrains.buildServer.configs.kotlin.failureConditions.BuildFailureOnMetric
+import jetbrains.buildServer.configs.kotlin.failureConditions.failOnMetricChange
+import jetbrains.buildServer.configs.kotlin.project
+import jetbrains.buildServer.configs.kotlin.triggers.schedule
+import jetbrains.buildServer.configs.kotlin.triggers.vcs
+import jetbrains.buildServer.configs.kotlin.version
 
 /*
 The settings script is an entry point for defining a TeamCity
@@ -32,29 +38,28 @@ To debug in IntelliJ Idea, open the 'Maven Projects' tool window (View
 'Debug' option is available in the context menu for the task.
 */
 
-version = "2020.2"
+version = "2023.05"
 
 val targets = arrayOf(
-        "linux/amd64/1.18",
-        "linux/amd64/1.19",
-        "linux/amd64/1.20",
+        "linux/amd64/1.22",
+        "linux/amd64/1.23",
+        "linux/amd64/1.24",
         "linux/amd64/tip",
 
-        "linux/386/1.20",
+        "linux/386/1.24",
 
-        "linux/arm64/1.20",
+        "linux/arm64/1.24",
         "linux/arm64/tip",
 
-        "windows/amd64/1.20",
+        "linux/ppc64le/1.24",
+
+        "windows/amd64/1.24",
         "windows/amd64/tip",
 
-        "windows/arm64/1.20",
-        "windows/arm64/tip",
-
-        "mac/amd64/1.20",
+        "mac/amd64/1.24",
         "mac/amd64/tip",
 
-        "mac/arm64/1.20",
+        "mac/arm64/1.24",
         "mac/arm64/tip"
 )
 
@@ -67,10 +72,15 @@ project {
         test.os
     }.distinct().forEach { os ->
         subProject(OSProject(os, tests.filter { test ->
-            test.os == os
+            test.os == os && test.version != "tip"
         }))
     }
-    buildType(AggregatorBuild(tests))
+    subProject(TipProject(tests.filter { test ->
+        test.version == "tip"
+    }))
+    buildType(AggregatorBuild(tests.filter { test ->
+        test.version != "tip"
+    }))
     params {
         param("teamcity.ui.settings.readOnly", "true")
     }
@@ -123,6 +133,15 @@ class AggregatorBuild(tests: Collection<BuildType>) : BuildType({
     }
 })
 
+class TipProject(tests: List<TestBuild>) : Project({
+    id = AbsoluteId("Delve_tip")
+    name = "Tip"
+
+    tests.forEach { test ->
+        buildType(test)
+    }
+})
+
 class OSProject(os: String, tests: List<TestBuild>) : Project({
     id = AbsoluteId("Delve_$os")
     name = os.capitalize()
@@ -145,12 +164,32 @@ class ArchProject(os: String, arch: String, tests: List<TestBuild>) : Project({
     }
 })
 
-class TestBuild(val os: String, val arch: String, version: String, buildId: AbsoluteId) : BuildType({
+class TestBuild(val os: String, val arch: String, val version: String, buildId: AbsoluteId) : BuildType({
     id = buildId
-    name = version
+    name = if (version == "tip") "${os}_${arch}_tip" else version
 
     vcs {
         root(DslContext.settingsRoot)
+        branchFilter = if (version == "tip") {
+            """
+                +:*
+                -:pull/*
+                """.trimIndent()
+        } else {
+            "+:*"
+        }
+    }
+
+    if (version == "tip") {
+        triggers {
+            schedule {
+                schedulingPolicy = daily {
+                    hour = 23
+                }
+                withPendingChangesOnly = true
+                triggerBuild = always()
+            }
+        }
     }
 
     failureConditions {
@@ -176,6 +215,12 @@ class TestBuild(val os: String, val arch: String, version: String, buildId: Abso
                         arch
                     }
                 }
+                val dockerPlatformArch = when (arch) {
+                    "arm64" -> "arm64/v8"
+                    else -> {
+                        dockerArch
+                    }
+                }
                 dockerCommand {
                     name = "Pull Ubuntu"
                     commandType = other {
@@ -192,6 +237,7 @@ class TestBuild(val os: String, val arch: String, version: String, buildId: Abso
                         --env TEAMCITY_VERSION=${'$'}TEAMCITY_VERSION
                         --env CI=true
                         --privileged
+                        --platform linux/$dockerPlatformArch
                         $dockerArch/ubuntu:20.04
                         /delve/_scripts/test_linux.sh ${"go$version"} $arch
                     """.trimIndent()
@@ -204,7 +250,7 @@ class TestBuild(val os: String, val arch: String, version: String, buildId: Abso
                     scriptMode = file {
                         path = "_scripts/test_windows.ps1"
                     }
-                    param("jetbrains_powershell_scriptArguments", "-version ${"go$version"} -arch $arch -binDir %teamcity.build.systemDir%")
+                    param("jetbrains_powershell_scriptArguments", "-version ${"go$version"} -arch $arch")
                 }
             }
             "mac" -> {
@@ -221,6 +267,8 @@ class TestBuild(val os: String, val arch: String, version: String, buildId: Abso
         when (arch) {
             "386", "amd64" -> equals("teamcity.agent.jvm.os.arch", if (os == "mac") "x86_64" else "amd64")
             "arm64" -> equals("teamcity.agent.jvm.os.arch", "aarch64")
+            "ppc64le" -> equals("teamcity.agent.jvm.os.arch", "ppc64le")
+            // "riscv64" -> equals("teamcity.agent.jvm.os.arch", "riscv64") // The riscv64 needs a builder
         }
         when (os) {
             "linux" -> {
