@@ -1,18 +1,17 @@
 package evalop
 
 import (
-	"bytes"
 	"errors"
 	"fmt"
 	"go/ast"
 	"go/constant"
 	"go/parser"
-	"go/printer"
 	"go/scanner"
 	"go/token"
 	"strconv"
 	"strings"
 
+	"github.com/go-delve/delve/pkg/astutil"
 	"github.com/go-delve/delve/pkg/dwarf/godwarf"
 	"github.com/go-delve/delve/pkg/dwarf/reader"
 )
@@ -21,29 +20,46 @@ var (
 	ErrFuncCallNotAllowed = errors.New("function calls not allowed without using 'call'")
 )
 
+const (
+	BreakpointHitCountVarNamePackage   = "delve"
+	BreakpointHitCountVarName          = "bphitcount"
+	BreakpointHitCountVarNameQualified = BreakpointHitCountVarNamePackage + "." + BreakpointHitCountVarName
+	DebugPinnerFunctionName            = "runtime.debugPinnerV1"
+)
+
 type compileCtx struct {
 	evalLookup
 	ops        []Op
 	allowCalls bool
 	curCall    int
+	flags      Flags
+	pinnerUsed bool
+	hasCalls   bool
 }
 
 type evalLookup interface {
 	FindTypeExpr(ast.Expr) (godwarf.Type, error)
-	HasLocal(string) bool
-	HasGlobal(string, string) bool
 	HasBuiltin(string) bool
-	LookupRegisterName(string) (int, bool)
 }
 
+// Flags describes flags used to control Compile and CompileAST
+type Flags uint8
+
+const (
+	CanSet         Flags = 1 << iota // Assignment is allowed
+	HasDebugPinner                   // runtime.debugPinner is available
+)
+
 // CompileAST compiles the expression t into a list of instructions.
-func CompileAST(lookup evalLookup, t ast.Expr) ([]Op, error) {
-	ctx := &compileCtx{evalLookup: lookup, allowCalls: true}
-	err := ctx.compileAST(t)
+func CompileAST(lookup evalLookup, t ast.Expr, flags Flags) ([]Op, error) {
+	ctx := &compileCtx{evalLookup: lookup, allowCalls: true, flags: flags}
+	err := ctx.compileAST(t, true)
 	if err != nil {
 		return nil, err
 	}
 
+	ctx.compileDebugPinnerSetupTeardown()
+
 	err = ctx.depthCheck(1)
 	if err != nil {
 		return ctx.ops, err
@@ -53,18 +69,18 @@ func CompileAST(lookup evalLookup, t ast.Expr) ([]Op, error) {
 
 // Compile compiles the expression expr into a list of instructions.
 // If canSet is true expressions like "x = y" are also accepted.
-func Compile(lookup evalLookup, expr string, canSet bool) ([]Op, error) {
+func Compile(lookup evalLookup, expr string, flags Flags) ([]Op, error) {
 	t, err := parser.ParseExpr(expr)
 	if err != nil {
-		if canSet {
+		if flags&CanSet != 0 {
 			eqOff, isAs := isAssignment(err)
 			if isAs {
-				return CompileSet(lookup, expr[:eqOff], expr[eqOff+1:])
+				return CompileSet(lookup, expr[:eqOff], expr[eqOff+1:], flags)
 			}
 		}
 		return nil, err
 	}
-	return CompileAST(lookup, t)
+	return CompileAST(lookup, t, flags)
 }
 
 func isAssignment(err error) (int, bool) {
@@ -77,7 +93,7 @@ func isAssignment(err error) (int, bool) {
 
 // CompileSet compiles the expression setting lhexpr to rhexpr into a list of
 // instructions.
-func CompileSet(lookup evalLookup, lhexpr, rhexpr string) ([]Op, error) {
+func CompileSet(lookup evalLookup, lhexpr, rhexpr string, flags Flags) ([]Op, error) {
 	lhe, err := parser.ParseExpr(lhexpr)
 	if err != nil {
 		return nil, err
@@ -87,23 +103,25 @@ func CompileSet(lookup evalLookup, lhexpr, rhexpr string) ([]Op, error) {
 		return nil, err
 	}
 
-	ctx := &compileCtx{evalLookup: lookup, allowCalls: true}
-	err = ctx.compileAST(rhe)
+	ctx := &compileCtx{evalLookup: lookup, allowCalls: true, flags: flags}
+	err = ctx.compileAST(rhe, false)
 	if err != nil {
 		return nil, err
 	}
 
-	if isStringLiteral(rhe) {
+	if isStringLiteralThatNeedsAlloc(rhe) {
 		ctx.compileAllocLiteralString()
 	}
 
-	err = ctx.compileAST(lhe)
+	err = ctx.compileAST(lhe, false)
 	if err != nil {
 		return nil, err
 	}
 
 	ctx.pushOp(&SetValue{lhe: lhe, Rhe: rhe})
 
+	ctx.compileDebugPinnerSetupTeardown()
+
 	err = ctx.depthCheck(0)
 	if err != nil {
 		return ctx.ops, err
@@ -112,9 +130,101 @@ func CompileSet(lookup evalLookup, lhexpr, rhexpr string) ([]Op, error) {
 }
 
 func (ctx *compileCtx) compileAllocLiteralString() {
-	ctx.pushOp(&CallInjectionAllocString{Phase: 0})
-	ctx.pushOp(&CallInjectionAllocString{Phase: 1})
-	ctx.pushOp(&CallInjectionAllocString{Phase: 2})
+	jmp := &Jump{When: JumpIfAllocStringChecksFail}
+	ctx.pushOp(jmp)
+
+	ctx.compileSpecialCall("runtime.mallocgc", []ast.Expr{
+		&ast.BasicLit{Kind: token.INT, Value: "0"},
+		&ast.Ident{Name: "nil"},
+		&ast.Ident{Name: "false"},
+	}, []Op{
+		&PushLen{},
+		&PushNil{},
+		&PushConst{constant.MakeBool(false)},
+	}, specialCallDoPinning|specialCallIsStringAlloc)
+
+	ctx.pushOp(&ConvertAllocToString{})
+	jmp.Target = len(ctx.ops)
+}
+
+type specialCallFlags uint8
+
+const (
+	specialCallDoPinning specialCallFlags = 1 << iota
+	specialCallIsStringAlloc
+	specialCallComplainAboutStringAlloc
+)
+
+func (ctx *compileCtx) compileSpecialCall(fnname string, argAst []ast.Expr, args []Op, flags specialCallFlags) {
+	doPinning := flags&specialCallDoPinning != 0
+
+	id := ctx.curCall
+	ctx.curCall++
+	ctx.pushOp(&CallInjectionStartSpecial{
+		id:     id,
+		FnName: fnname,
+		ArgAst: argAst,
+
+		ComplainAboutStringAlloc: flags&specialCallComplainAboutStringAlloc != 0})
+	ctx.pushOp(&CallInjectionSetTarget{id: id})
+
+	for i := range args {
+		if args[i] != nil {
+			ctx.pushOp(args[i])
+		}
+		ctx.pushOp(&CallInjectionCopyArg{id: id, ArgNum: i})
+	}
+
+	doPinning = doPinning && (ctx.flags&HasDebugPinner != 0)
+
+	if doPinning {
+		ctx.pinnerUsed = true
+		if flags&specialCallIsStringAlloc == 0 {
+			ctx.hasCalls = true
+		}
+	}
+
+	ctx.pushOp(&CallInjectionComplete{id: id, DoPinning: doPinning})
+
+	if doPinning {
+		ctx.compilePinningLoop(id)
+	}
+}
+
+func (ctx *compileCtx) compileDebugPinnerSetupTeardown() {
+	if !ctx.pinnerUsed {
+		return
+	}
+
+	// Prepend debug pinner allocation
+	mainops := ctx.ops
+	ctx.ops = []Op{}
+	flags := specialCallFlags(0)
+	if !ctx.hasCalls {
+		flags = specialCallComplainAboutStringAlloc
+	}
+	ctx.compileSpecialCall(DebugPinnerFunctionName, []ast.Expr{}, []Op{}, flags)
+	ctx.pushOp(&SetDebugPinner{})
+
+	// Adjust jump destinations
+	for _, op := range mainops {
+		switch op := op.(type) {
+		case *Jump:
+			op.Target += len(ctx.ops)
+		}
+	}
+
+	ctx.ops = append(ctx.ops, mainops...)
+
+	// Append debug pinner deallocation
+	ctx.compileSpecialCall("runtime.(*Pinner).Unpin", []ast.Expr{
+		&ast.Ident{Name: "debugPinner"},
+	}, []Op{
+		&PushDebugPinner{},
+	}, 0)
+	ctx.pushOp(&Pop{})
+	ctx.pushOp(&PushNil{})
+	ctx.pushOp(&SetDebugPinner{})
 }
 
 func (ctx *compileCtx) pushOp(op Op) {
@@ -146,6 +256,8 @@ func (ctx *compileCtx) depthCheck(endDepth int) error {
 		}
 	}
 
+	debugPinnerSeen := false
+
 	for i, op := range ctx.ops {
 		npop, npush := op.depthCheck()
 		if depth[i] < npop {
@@ -153,8 +265,15 @@ func (ctx *compileCtx) depthCheck(endDepth int) error {
 		}
 		d := depth[i] - npop + npush
 		checkAndSet(i+1, d)
-		if jmp, _ := op.(*Jump); jmp != nil {
-			checkAndSet(jmp.Target, d)
+		switch op := op.(type) {
+		case *Jump:
+			checkAndSet(op.Target, d)
+		case *CallInjectionStartSpecial:
+			debugPinnerSeen = true
+		case *CallInjectionComplete:
+			if op.DoPinning && !debugPinnerSeen {
+				err = fmt.Errorf("internal debugger error: pinning call injection seen before call to %s at instruction %d", DebugPinnerFunctionName, i)
+			}
 		}
 		if err != nil {
 			return err
@@ -167,17 +286,17 @@ func (ctx *compileCtx) depthCheck(endDepth int) error {
 	return nil
 }
 
-func (ctx *compileCtx) compileAST(t ast.Expr) error {
+func (ctx *compileCtx) compileAST(t ast.Expr, toplevel bool) error {
 	switch node := t.(type) {
 	case *ast.CallExpr:
-		return ctx.compileTypeCastOrFuncCall(node)
+		return ctx.compileTypeCastOrFuncCall(node, toplevel)
 
 	case *ast.Ident:
 		return ctx.compileIdent(node)
 
 	case *ast.ParenExpr:
 		// otherwise just eval recursively
-		return ctx.compileAST(node.X)
+		return ctx.compileAST(node.X, false)
 
 	case *ast.SelectorExpr: // <expression>.<identifier>
 		switch x := node.X.(type) {
@@ -192,15 +311,14 @@ func (ctx *compileCtx) compileAST(t ast.Expr) error {
 			case x.Name == "runtime" && node.Sel.Name == "threadid":
 				ctx.pushOp(&PushThreadID{})
 
-			case ctx.HasLocal(x.Name):
-				ctx.pushOp(&PushLocal{Name: x.Name})
-				ctx.pushOp(&Select{node.Sel.Name})
+			case x.Name == "runtime" && node.Sel.Name == "rangeParentOffset":
+				ctx.pushOp(&PushRangeParentOffset{})
 
-			case ctx.HasGlobal(x.Name, node.Sel.Name):
-				ctx.pushOp(&PushPackageVar{x.Name, node.Sel.Name})
+			case x.Name == BreakpointHitCountVarNamePackage && node.Sel.Name == BreakpointHitCountVarName:
+				ctx.pushOp(&PushBreakpointHitCount{})
 
 			default:
-				return ctx.compileUnary(node.X, &Select{node.Sel.Name})
+				ctx.pushOp(&PushPackageVarOrSelect{Name: x.Name, Sel: node.Sel.Name})
 			}
 
 		case *ast.CallExpr:
@@ -229,15 +347,10 @@ func (ctx *compileCtx) compileAST(t ast.Expr) error {
 			if err != nil {
 				return err
 			}
-			if ctx.HasGlobal(s, node.Sel.Name) {
-				ctx.pushOp(&PushPackageVar{s, node.Sel.Name})
-				return nil
-			}
-			return ctx.compileUnary(node.X, &Select{node.Sel.Name})
+			ctx.pushOp(&PushPackageVarOrSelect{Name: s, Sel: node.Sel.Name, NameIsString: true})
 
 		default:
 			return ctx.compileUnary(node.X, &Select{node.Sel.Name})
-
 		}
 
 	case *ast.TypeAssertExpr: // <expression>.(<type>)
@@ -248,7 +361,7 @@ func (ctx *compileCtx) compileAST(t ast.Expr) error {
 
 	case *ast.SliceExpr:
 		if node.Slice3 {
-			return fmt.Errorf("3-index slice expressions not supported")
+			return errors.New("3-index slice expressions not supported")
 		}
 		return ctx.compileReslice(node)
 
@@ -296,10 +409,10 @@ func (ctx *compileCtx) compileAST(t ast.Expr) error {
 	return nil
 }
 
-func (ctx *compileCtx) compileTypeCastOrFuncCall(node *ast.CallExpr) error {
+func (ctx *compileCtx) compileTypeCastOrFuncCall(node *ast.CallExpr, toplevel bool) error {
 	if len(node.Args) != 1 {
 		// Things that have more or less than one argument are always function calls.
-		return ctx.compileFunctionCall(node)
+		return ctx.compileFunctionCall(node, toplevel)
 	}
 
 	ambiguous := func() error {
@@ -307,9 +420,9 @@ func (ctx *compileCtx) compileTypeCastOrFuncCall(node *ast.CallExpr) error {
 		// evaluated then try to treat it as a function call, otherwise try the
 		// type cast.
 		ctx2 := &compileCtx{evalLookup: ctx.evalLookup}
-		err0 := ctx2.compileAST(node.Fun)
+		err0 := ctx2.compileAST(node.Fun, false)
 		if err0 == nil {
-			return ctx.compileFunctionCall(node)
+			return ctx.compileFunctionCall(node, toplevel)
 		}
 		return ctx.compileTypeCast(node, err0)
 	}
@@ -337,15 +450,12 @@ func (ctx *compileCtx) compileTypeCastOrFuncCall(node *ast.CallExpr) error {
 			}
 			return ambiguous()
 		}
-		return ctx.compileFunctionCall(node)
+		return ctx.compileFunctionCall(node, toplevel)
 	case *ast.Ident:
-		if ctx.HasBuiltin(n.Name) {
-			return ctx.compileFunctionCall(node)
+		if typ, _ := ctx.FindTypeExpr(n); typ != nil {
+			return ctx.compileTypeCast(node, fmt.Errorf("could not find symbol value for %s", n.Name))
 		}
-		if ctx.HasGlobal("", n.Name) || ctx.HasLocal(n.Name) {
-			return ctx.compileFunctionCall(node)
-		}
-		return ctx.compileTypeCast(node, fmt.Errorf("could not find symbol value for %s", n.Name))
+		return ctx.compileFunctionCall(node, toplevel)
 	case *ast.IndexExpr:
 		// Ambiguous, could be a parametric type
 		switch n.X.(type) {
@@ -355,20 +465,20 @@ func (ctx *compileCtx) compileTypeCastOrFuncCall(node *ast.CallExpr) error {
 			if err == nil || err != reader.ErrTypeNotFound {
 				return err
 			}
-			return ctx.compileFunctionCall(node)
+			return ctx.compileFunctionCall(node, toplevel)
 		default:
-			return ctx.compileFunctionCall(node)
+			return ctx.compileFunctionCall(node, toplevel)
 		}
-	case *astIndexListExpr:
+	case *ast.IndexListExpr:
 		return ctx.compileTypeCast(node, nil)
 	default:
 		// All other expressions must be function calls
-		return ctx.compileFunctionCall(node)
+		return ctx.compileFunctionCall(node, toplevel)
 	}
 }
 
 func (ctx *compileCtx) compileTypeCast(node *ast.CallExpr, ambiguousErr error) error {
-	err := ctx.compileAST(node.Args[0])
+	err := ctx.compileAST(node.Args[0], false)
 	if err != nil {
 		return err
 	}
@@ -378,7 +488,7 @@ func (ctx *compileCtx) compileTypeCast(node *ast.CallExpr, ambiguousErr error) e
 	// remove all enclosing parenthesis from the type name
 	fnnode = removeParen(fnnode)
 
-	targetTypeStr := exprToString(removeParen(node.Fun))
+	targetTypeStr := astutil.ExprToString(removeParen(node.Fun))
 	styp, err := ctx.FindTypeExpr(fnnode)
 	if err != nil {
 		switch targetTypeStr {
@@ -388,7 +498,7 @@ func (ctx *compileCtx) compileTypeCast(node *ast.CallExpr, ambiguousErr error) e
 			styp = godwarf.FakeSliceType(godwarf.FakeBasicType("int", 32))
 		default:
 			if ambiguousErr != nil && err == reader.ErrTypeNotFound {
-				return fmt.Errorf("could not evaluate function or type %s: %v", exprToString(node.Fun), ambiguousErr)
+				return fmt.Errorf("could not evaluate function or type %s: %v", astutil.ExprToString(node.Fun), ambiguousErr)
 			}
 			return err
 		}
@@ -400,7 +510,7 @@ func (ctx *compileCtx) compileTypeCast(node *ast.CallExpr, ambiguousErr error) e
 
 func (ctx *compileCtx) compileBuiltinCall(builtin string, args []ast.Expr) error {
 	for _, arg := range args {
-		err := ctx.compileAST(arg)
+		err := ctx.compileAST(arg, false)
 		if err != nil {
 			return err
 		}
@@ -410,30 +520,12 @@ func (ctx *compileCtx) compileBuiltinCall(builtin string, args []ast.Expr) error
 }
 
 func (ctx *compileCtx) compileIdent(node *ast.Ident) error {
-	switch {
-	case ctx.HasLocal(node.Name):
-		ctx.pushOp(&PushLocal{Name: node.Name})
-	case ctx.HasGlobal("", node.Name):
-		ctx.pushOp(&PushPackageVar{"", node.Name})
-	case node.Name == "true" || node.Name == "false":
-		ctx.pushOp(&PushConst{constant.MakeBool(node.Name == "true")})
-	case node.Name == "nil":
-		ctx.pushOp(&PushNil{})
-	default:
-		found := false
-		if regnum, ok := ctx.LookupRegisterName(node.Name); ok {
-			ctx.pushOp(&PushRegister{regnum, node.Name})
-			found = true
-		}
-		if !found {
-			return fmt.Errorf("could not find symbol value for %s", node.Name)
-		}
-	}
+	ctx.pushOp(&PushIdent{node.Name})
 	return nil
 }
 
 func (ctx *compileCtx) compileUnary(expr ast.Expr, op Op) error {
-	err := ctx.compileAST(expr)
+	err := ctx.compileAST(expr, false)
 	if err != nil {
 		return err
 	}
@@ -442,7 +534,7 @@ func (ctx *compileCtx) compileUnary(expr ast.Expr, op Op) error {
 }
 
 func (ctx *compileCtx) compileTypeAssert(node *ast.TypeAssertExpr) error {
-	err := ctx.compileAST(node.X)
+	err := ctx.compileAST(node.X, false)
 	if err != nil {
 		return err
 	}
@@ -462,14 +554,14 @@ func (ctx *compileCtx) compileTypeAssert(node *ast.TypeAssertExpr) error {
 }
 
 func (ctx *compileCtx) compileBinary(a, b ast.Expr, sop *Jump, op Op) error {
-	err := ctx.compileAST(a)
+	err := ctx.compileAST(a, false)
 	if err != nil {
 		return err
 	}
 	if sop != nil {
 		ctx.pushOp(sop)
 	}
-	err = ctx.compileAST(b)
+	err = ctx.compileAST(b, false)
 	if err != nil {
 		return err
 	}
@@ -478,34 +570,41 @@ func (ctx *compileCtx) compileBinary(a, b ast.Expr, sop *Jump, op Op) error {
 }
 
 func (ctx *compileCtx) compileReslice(node *ast.SliceExpr) error {
-	err := ctx.compileAST(node.X)
+	err := ctx.compileAST(node.X, false)
 	if err != nil {
 		return err
 	}
 
+	trustLen := true
 	hasHigh := false
 	if node.High != nil {
 		hasHigh = true
-		err = ctx.compileAST(node.High)
+		err = ctx.compileAST(node.High, false)
 		if err != nil {
 			return err
 		}
+		_, isbasiclit := node.High.(*ast.BasicLit)
+		trustLen = trustLen && isbasiclit
+	} else {
+		trustLen = false
 	}
 
 	if node.Low != nil {
-		err = ctx.compileAST(node.Low)
+		err = ctx.compileAST(node.Low, false)
 		if err != nil {
 			return err
 		}
+		_, isbasiclit := node.Low.(*ast.BasicLit)
+		trustLen = trustLen && isbasiclit
 	} else {
 		ctx.pushOp(&PushConst{constant.MakeInt64(0)})
 	}
 
-	ctx.pushOp(&Reslice{Node: node, HasHigh: hasHigh})
+	ctx.pushOp(&Reslice{Node: node, HasHigh: hasHigh, TrustLen: trustLen})
 	return nil
 }
 
-func (ctx *compileCtx) compileFunctionCall(node *ast.CallExpr) error {
+func (ctx *compileCtx) compileFunctionCall(node *ast.CallExpr, toplevel bool) error {
 	if fnnode, ok := node.Fun.(*ast.Ident); ok {
 		if ctx.HasBuiltin(fnnode.Name) {
 			return ctx.compileBuiltinCall(fnnode.Name, node.Args)
@@ -518,10 +617,20 @@ func (ctx *compileCtx) compileFunctionCall(node *ast.CallExpr) error {
 	id := ctx.curCall
 	ctx.curCall++
 
+	if ctx.flags&HasDebugPinner != 0 {
+		return ctx.compileFunctionCallWithPinning(node, id, toplevel)
+	}
+
+	return ctx.compileFunctionCallNoPinning(node, id)
+}
+
+// compileFunctionCallNoPinning compiles a function call when runtime.debugPinner is
+// not available in the target.
+func (ctx *compileCtx) compileFunctionCallNoPinning(node *ast.CallExpr, id int) error {
 	oldAllowCalls := ctx.allowCalls
 	oldOps := ctx.ops
 	ctx.allowCalls = false
-	err := ctx.compileAST(node.Fun)
+	err := ctx.compileAST(node.Fun, false)
 	ctx.allowCalls = oldAllowCalls
 	hasFunc := false
 	if err != nil {
@@ -541,7 +650,7 @@ func (ctx *compileCtx) compileFunctionCall(node *ast.CallExpr) error {
 		ctx.pushOp(jmpif)
 	}
 	ctx.pushOp(&Pop{})
-	err = ctx.compileAST(node.Fun)
+	err = ctx.compileAST(node.Fun, false)
 	if err != nil {
 		return err
 	}
@@ -552,11 +661,11 @@ func (ctx *compileCtx) compileFunctionCall(node *ast.CallExpr) error {
 	ctx.pushOp(&CallInjectionSetTarget{id: id})
 
 	for i, arg := range node.Args {
-		err := ctx.compileAST(arg)
+		err := ctx.compileAST(arg, false)
 		if err != nil {
-			return fmt.Errorf("error evaluating %q as argument %d in function %s: %v", exprToString(arg), i+1, exprToString(node.Fun), err)
+			return fmt.Errorf("error evaluating %q as argument %d in function %s: %v", astutil.ExprToString(arg), i+1, astutil.ExprToString(node.Fun), err)
 		}
-		if isStringLiteral(arg) {
+		if isStringLiteralThatNeedsAlloc(arg) {
 			ctx.compileAllocLiteralString()
 		}
 		ctx.pushOp(&CallInjectionCopyArg{id: id, ArgNum: i, ArgExpr: arg})
@@ -567,6 +676,66 @@ func (ctx *compileCtx) compileFunctionCall(node *ast.CallExpr) error {
 	return nil
 }
 
+// compileFunctionCallWithPinning compiles a function call when runtime.debugPinner
+// is available in the target.
+func (ctx *compileCtx) compileFunctionCallWithPinning(node *ast.CallExpr, id int, toplevel bool) error {
+	if !toplevel {
+		ctx.pinnerUsed = true
+	}
+	ctx.hasCalls = true
+
+	err := ctx.compileAST(node.Fun, false)
+	if err != nil {
+		return err
+	}
+
+	for i, arg := range node.Args {
+		err := ctx.compileAST(arg, false)
+		if isStringLiteralThatNeedsAlloc(arg) {
+			ctx.compileAllocLiteralString()
+		}
+		if err != nil {
+			return fmt.Errorf("error evaluating %q as argument %d in function %s: %v", astutil.ExprToString(arg), i+1, astutil.ExprToString(node.Fun), err)
+		}
+	}
+
+	ctx.pushOp(&Roll{len(node.Args)})
+	ctx.pushOp(&CallInjectionStart{HasFunc: true, id: id, Node: node})
+	ctx.pushOp(&Pop{})
+	ctx.pushOp(&CallInjectionSetTarget{id: id})
+
+	for i := len(node.Args) - 1; i >= 0; i-- {
+		arg := node.Args[i]
+		ctx.pushOp(&CallInjectionCopyArg{id: id, ArgNum: i, ArgExpr: arg})
+	}
+
+	ctx.pushOp(&CallInjectionComplete{id: id, DoPinning: !toplevel})
+
+	if !toplevel {
+		ctx.compilePinningLoop(id)
+	}
+
+	return nil
+}
+
+func (ctx *compileCtx) compilePinningLoop(id int) {
+	loopStart := len(ctx.ops)
+	jmp := &Jump{When: JumpIfPinningDone}
+	ctx.pushOp(jmp)
+	ctx.pushOp(&PushPinAddress{})
+	ctx.compileSpecialCall("runtime.(*Pinner).Pin", []ast.Expr{
+		&ast.Ident{Name: "debugPinner"},
+		&ast.Ident{Name: "pinAddress"},
+	}, []Op{
+		&PushDebugPinner{},
+		nil,
+	}, 0)
+	ctx.pushOp(&Pop{})
+	ctx.pushOp(&Jump{When: JumpAlways, Target: loopStart})
+	jmp.Target = len(ctx.ops)
+	ctx.pushOp(&CallInjectionComplete2{id: id})
+}
+
 func Listing(depth []int, ops []Op) string {
 	if depth == nil {
 		depth = make([]int, len(ops)+1)
@@ -578,16 +747,16 @@ func Listing(depth []int, ops []Op) string {
 	return buf.String()
 }
 
-func isStringLiteral(expr ast.Expr) bool {
+func isStringLiteralThatNeedsAlloc(expr ast.Expr) bool {
 	switch expr := expr.(type) {
 	case *ast.BasicLit:
-		return expr.Kind == token.STRING
+		return expr.Kind == token.STRING && expr.Value != `""`
 	case *ast.BinaryExpr:
 		if expr.Op == token.ADD {
-			return isStringLiteral(expr.X) && isStringLiteral(expr.Y)
+			return isStringLiteralThatNeedsAlloc(expr.X) && isStringLiteralThatNeedsAlloc(expr.Y)
 		}
 	case *ast.ParenExpr:
-		return isStringLiteral(expr.X)
+		return isStringLiteralThatNeedsAlloc(expr.X)
 	}
 	return false
 }
@@ -602,9 +771,3 @@ func removeParen(n ast.Expr) ast.Expr {
 	}
 	return n
 }
-
-func exprToString(t ast.Expr) string {
-	var buf bytes.Buffer
-	printer.Fprint(&buf, token.NewFileSet(), t)
-	return buf.String()
-}
