@@ -299,7 +299,7 @@ func (it *stackIterator) switchToGoroutineStack() {
 	it.pc = it.g.PC
 	it.regs.Reg(it.regs.SPRegNum).Uint64Val = it.g.SP
 	it.regs.AddReg(it.regs.BPRegNum, op.DwarfRegisterFromUint64(it.g.BP))
-	if it.bi.Arch.Name == "arm64" || it.bi.Arch.Name == "ppc64le" {
+	if it.bi.Arch.Name == "arm64" || it.bi.Arch.Name == "ppc64le" || it.bi.Arch.Name == "riscv64" || it.bi.Arch.Name == "loong64" {
 		it.regs.Reg(it.regs.LRRegNum).Uint64Val = it.g.LR
 	}
 }
@@ -382,15 +382,31 @@ func (it *stackIterator) stacktrace(depth int) ([]Stackframe, error) {
 		return nil, errors.New("negative maximum stack depth")
 	}
 	frames := make([]Stackframe, 0, depth+1)
-	it.stacktraceFunc(func(frame Stackframe) bool {
+	f := func(frame Stackframe) bool {
 		frames = append(frames, frame)
 		return len(frames) < depth+1
-	})
+	}
+	it.stacktraceFunc(f)
+	if it.Err() != nil && len(frames) == 1 && it.g != nil && frames[0].SystemStack && (it.opts&StacktraceSimple == 0) {
+		// If we can't continue from the first frame, and it was on a system stack
+		// and we have a goroutine which we are allowed to switch to then switch
+		// to it and continue the stacktrace from there.
+		// This improves stacktraces produced on Windows by WER where the first
+		// thread will be executing a system function from which we can't continue
+		// to trace.
+		// See #3824.
+		it.err = nil
+		it.opts |= StacktraceG
+		it.stacktraceFunc(f)
+	}
+
 	if err := it.Err(); err != nil {
 		if len(frames) == 0 {
 			return nil, err
 		}
+
 		frames = append(frames, Stackframe{Err: err})
+
 	}
 	return frames, nil
 }
@@ -564,7 +580,7 @@ func (it *stackIterator) advanceRegs() (callFrameRegs op.DwarfRegisters, ret uin
 		}
 	}
 
-	if it.bi.Arch.Name == "arm64" || it.bi.Arch.Name == "ppc64le" {
+	if it.bi.Arch.Name == "arm64" || it.bi.Arch.Name == "ppc64le" || it.bi.Arch.Name == "riscv64" || it.bi.Arch.Name == "loong64" {
 		if ret == 0 && it.regs.Reg(it.regs.LRRegNum) != nil {
 			ret = it.regs.Reg(it.regs.LRRegNum).Uint64Val
 		}
@@ -964,7 +980,7 @@ func rangeFuncStackTrace(tgt *Target, g *G) ([]Stackframe, error) {
 		if fr.SystemStack {
 			return false
 		}
-		if closurePtr == 0 && optimized(fr.Call.Fn) {
+		if closurePtr == 0 && optimized(fr.Call.Fn) || frames[len(frames)-1].Inlined {
 			return true
 		}
 		if closurePtr < 0 {
@@ -1021,7 +1037,7 @@ func rangeFuncStackTrace(tgt *Target, g *G) ([]Stackframe, error) {
 			if rangeParent == nil {
 				stop = true
 			}
-			if !optimized(fr.Call.Fn) && closurePtr == 0 {
+			if !optimized(fr.Call.Fn) && !fr.Inlined && closurePtr == 0 {
 				stop = true
 			}
 			if stop {
@@ -1042,6 +1058,11 @@ func rangeFuncStackTrace(tgt *Target, g *G) ([]Stackframe, error) {
 					stage = doneStage
 					return false
 				}
+			} else if frames[len(frames)-1].Inlined && !fr.Inlined && closurePtr == 0 {
+				frames = nil
+				addRetFrame = false
+				stage = doneStage
+				return false
 			}
 		case lastFrameStage:
 			frames = append(frames, fr)
@@ -1053,7 +1074,7 @@ func rangeFuncStackTrace(tgt *Target, g *G) ([]Stackframe, error) {
 		return true
 	})
 	if it.Err() != nil {
-		return nil, err
+		return nil, it.Err()
 	}
 	if nonMonotonicSP {
 		return nil, errors.New("corrupted stack (SP not monotonically decreasing)")
