@@ -22,6 +22,12 @@ var EnableRace = flag.Bool("racetarget", false, "Enables race detector on inferi
 
 var runningWithFixtures bool
 
+var ldFlags string
+
+func init() {
+	ldFlags = os.Getenv("CGO_LDFLAGS")
+}
+
 // Fixture is a test binary.
 type Fixture struct {
 	// Name is the short name of the fixture.
@@ -158,7 +164,7 @@ func BuildFixture(name string, flags BuildFlags) Fixture {
 	if flags&BuildModeExternalLinker != 0 {
 		buildFlags = append(buildFlags, "-ldflags=-linkmode=external")
 	}
-	if ver.IsDevel() || ver.AfterOrEqual(goversion.GoVersion{Major: 1, Minor: 11, Rev: -1}) {
+	if ver.IsOldDevel() || ver.AfterOrEqual(goversion.GoVersion{Major: 1, Minor: 11, Rev: -1}) {
 		if flags&EnableDWZCompression != 0 {
 			buildFlags = append(buildFlags, "-ldflags=-compressdwarf=false")
 		}
@@ -206,14 +212,14 @@ func BuildFixture(name string, flags BuildFlags) Fixture {
 	return fixtures[fk]
 }
 
-// RunTestsWithFixtures will pre-compile test fixtures before running test
-// methods. Test binaries are deleted before exiting.
-func RunTestsWithFixtures(m *testing.M) int {
+// RunTestsWithFixtures sets the flag runningWithFixtures to compile fixtures on demand and runs tests with m.Run().
+// After the tests are run, it removes the fixtures and paths from PathsToRemove.
+func RunTestsWithFixtures(m *testing.M) {
 	runningWithFixtures = true
 	defer func() {
 		runningWithFixtures = false
 	}()
-	status := m.Run()
+	m.Run()
 
 	// Remove the fixtures.
 	for _, f := range fixtures {
@@ -231,7 +237,6 @@ func RunTestsWithFixtures(m *testing.M) int {
 			os.Remove(p)
 		}
 	}
-	return status
 }
 
 var recordingAllowed = map[string]bool{}
@@ -310,6 +315,12 @@ func MustSupportFunctionCalls(t *testing.T, testBackend string) {
 	if runtime.GOARCH == "386" {
 		t.Skip(fmt.Errorf("%s does not support FunctionCall for now", runtime.GOARCH))
 	}
+	if runtime.GOARCH == "riscv64" {
+		t.Skip(fmt.Errorf("%s does not support FunctionCall for now", runtime.GOARCH))
+	}
+	if runtime.GOARCH == "loong64" {
+		t.Skip(fmt.Errorf("%s does not support FunctionCall for now", runtime.GOARCH))
+	}
 	if runtime.GOARCH == "arm64" {
 		if !goversion.VersionAfterOrEqual(runtime.Version(), 1, 19) || runtime.GOOS == "windows" {
 			t.Skip("this version of Go does not support function calls")
@@ -382,10 +393,72 @@ func RegabiSupported() bool {
 	// Tracks regabiSupported variable in ParseGOEXPERIMENT internal/buildcfg/exp.go
 	switch {
 	case goversion.VersionAfterOrEqual(runtime.Version(), 1, 18):
-		return runtime.GOARCH == "amd64" || runtime.GOARCH == "arm64" || runtime.GOARCH == "ppc64le" || runtime.GOARCH == "ppc64"
+		return runtime.GOARCH == "amd64" || runtime.GOARCH == "arm64" || runtime.GOARCH == "ppc64le" || runtime.GOARCH == "ppc64" || runtime.GOARCH == "riscv64" || runtime.GOARCH == "loong64"
 	case goversion.VersionAfterOrEqual(runtime.Version(), 1, 17):
 		return runtime.GOARCH == "amd64" && (runtime.GOOS == "android" || runtime.GOOS == "linux" || runtime.GOOS == "darwin" || runtime.GOOS == "windows")
 	default:
 		return false
 	}
 }
+
+func ProjectRoot() string {
+	wd, err := os.Getwd()
+	if err != nil {
+		panic(err)
+	}
+
+	gopaths := strings.FieldsFunc(os.Getenv("GOPATH"), func(r rune) bool { return r == os.PathListSeparator })
+	for _, curpath := range gopaths {
+		// Detects "gopath mode" when GOPATH contains several paths ex. "d:\\dir\\gopath;f:\\dir\\gopath2"
+		if strings.Contains(wd, curpath) {
+			return filepath.Join(curpath, "src", "github.com", "go-delve", "delve")
+		}
+	}
+	val, err := exec.Command("go", "list", "-mod=", "-m", "-f", "{{ .Dir }}").Output()
+	if err != nil {
+		panic(err) // the Go tool was tested to work earlier
+	}
+	return strings.TrimSuffix(string(val), "\n")
+}
+
+func GetDlvBinary(t *testing.T) string {
+	// In case this was set in the environment
+	// from getDlvBinEBPF lets clear it here, so
+	// we can ensure we don't get build errors
+	// depending on the test ordering.
+	t.Setenv("CGO_LDFLAGS", ldFlags)
+	var tags []string
+	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
+		tags = []string{"-tags=exp.winarm64"}
+	}
+	if runtime.GOOS == "linux" && runtime.GOARCH == "ppc64le" {
+		tags = []string{"-tags=exp.linuxppc64le"}
+	}
+	if runtime.GOOS == "linux" && runtime.GOARCH == "riscv64" {
+		tags = []string{"-tags=exp.linuxriscv64"}
+	}
+	if runtime.GOOS == "linux" && runtime.GOARCH == "loong64" {
+		tags = []string{"-tags=exp.linuxloong64"}
+	}
+	return getDlvBinInternal(t, tags...)
+}
+
+func GetDlvBinaryEBPF(t *testing.T) string {
+	return getDlvBinInternal(t, "-tags", "ebpf")
+}
+
+func getDlvBinInternal(t *testing.T, goflags ...string) string {
+	dlvbin := filepath.Join(t.TempDir(), "dlv.exe")
+	args := append([]string{"build", "-o", dlvbin}, goflags...)
+	args = append(args, "github.com/go-delve/delve/cmd/dlv")
+
+	wd, _ := os.Getwd()
+	fmt.Printf("at %s %s\n", wd, goflags)
+
+	out, err := exec.Command("go", args...).CombinedOutput()
+	if err != nil {
+		t.Fatalf("go build -o %v github.com/go-delve/delve/cmd/dlv: %v\n%s", dlvbin, err, string(out))
+	}
+
+	return dlvbin
+}
